@@ -40,7 +40,7 @@
 #include <dpkg/triglib.h>
 
 const char *
-illegal_triggername(const char *p)
+trig_name_is_illegal(const char *p)
 {
 	int c;
 
@@ -55,37 +55,28 @@ illegal_triggername(const char *p)
 	return NULL;
 }
 
-/*========== recording triggers ==========*/
+/*========== Recording triggers. ==========*/
 
 static char *triggersdir, *triggersfilefile, *triggersnewfilefile;
 
-char *
-trig_get_triggersdir(const char *admindir)
-{
-	struct varbuf path = VARBUF_INIT;
-
-	varbufprintf(&path, "%s/%s", admindir, TRIGGERSDIR);
-
-	return varbuf_detach(&path);
-}
-
 static char *
 trig_get_filename(const char *dir, const char *filename)
 {
-	struct varbuf path = VARBUF_INIT;
+	char *path;
 
-	varbufprintf(&path, "%s/%s", dir, filename);
+	m_asprintf(&path, "%s/%s", dir, filename);
 
-	return varbuf_detach(&path);
+	return path;
 }
 
 static struct trig_hooks trigh;
 
-/*---------- noting trigger activation in memory ----------*/
+/*---------- Noting trigger activation in memory. ----------*/
 
-/* Called via trig_*activate* et al from:
+/*
+ * Called via trig_*activate* et al from:
  *   - trig_incorporate: reading of Unincorp (explicit trigger activations)
- *   - various places: processing start (`activate' in triggers ci file)
+ *   - various places: processing start (‘activate’ in triggers ci file)
  *   - namenodetouse: file triggers during unpack / remove
  *   - deferred_configure: file triggers during config file processing
  *
@@ -94,19 +85,21 @@ static struct trig_hooks trigh;
  * recorded (how would we know?) and (b) we don't enqueue them for
  * deferred processing in this run.
  *
- * We add the trigger to Triggers-Pending first.  This makes it
+ * We add the trigger to Triggers-Pending first. This makes it
  * harder to get into the state where Triggers-Awaited for aw lists
- * pend but Triggers-Pending for pend is empty.  (See also the
+ * pend but Triggers-Pending for pend is empty. (See also the
  * comment in deppossi_ok_found regarding this situation.)
  */
 
-/* aw might be NULL, and trig is not copied! */
+/*
+ * aw might be NULL.
+ * trig is not copied!
+ */
 static void
 trig_record_activation(struct pkginfo *pend, struct pkginfo *aw, const char *trig)
 {
 	if (pend->status < stat_triggersawaited)
-		/* Not interested then. */
-		return;
+		return; /* Not interested then. */
 
 	if (trig_note_pend(pend, trig))
 		modstatdb_note_ifwrite(pend);
@@ -122,9 +115,13 @@ trig_record_activation(struct pkginfo *pend, struct pkginfo *aw, const char *tri
 		}
 }
 
-/* NB that this is also called from fields.c where *pend is a temporary! */
+/*
+ * Note: This is also called from fields.c where *pend is a temporary!
+ *
+ * trig is not copied!
+ */
 bool
-trig_note_pend_core(struct pkginfo *pend, const char *trig /*not copied!*/)
+trig_note_pend_core(struct pkginfo *pend, const char *trig)
 {
 	struct trigpend *tp;
 
@@ -140,9 +137,14 @@ trig_note_pend_core(struct pkginfo *pend, const char *trig /*not copied!*/)
 	return true;
 }
 
-/* Returns: true for done, false for already noted. */
+/*
+ * trig is not copied!
+ *
+ * @retval true  For done.
+ * @retval false For already noted.
+ */
 bool
-trig_note_pend(struct pkginfo *pend, const char *trig /*not copied!*/)
+trig_note_pend(struct pkginfo *pend, const char *trig)
 {
 	if (!trig_note_pend_core(pend, trig))
 		return false;
@@ -153,9 +155,13 @@ trig_note_pend(struct pkginfo *pend, const char *trig /*not copied!*/)
 	return true;
 }
 
-/* Returns: true for done, false for already noted. */
-/* NB that this is called also from fields.c where *aw is a temporary
- * but pend is from findpackage()! */
+/*
+ * Note: This is called also from fields.c where *aw is a temporary
+ * but pend is from pkg_db_find()!
+ *
+ * @retval true  For done.
+ * @retval false For already noted.
+ */
 bool
 trig_note_aw(struct pkginfo *pend, struct pkginfo *aw)
 {
@@ -219,7 +225,7 @@ trig_enqueue_awaited_pend(struct pkginfo *pend)
  * while in modstatdb_note, and the package in triggers-pending had its
  * state modified but dpkg could not finish clearing the awaiters.
  *
- * XXX: possibly get rid of some of the checks done somewhere else for
+ * XXX: Possibly get rid of some of the checks done somewhere else for
  *      this condition at run-time.
  */
 void
@@ -238,7 +244,7 @@ trig_fixup_awaiters(enum modstatdb_rw cstatus)
 	trig_awaited_pend_head = NULL;
 }
 
-/*---------- generalised handling of trigger kinds ----------*/
+/*---------- Generalized handling of trigger kinds. ----------*/
 
 struct trigkindinfo {
 	/* Only for trig_activate_start. */
@@ -272,14 +278,15 @@ trig_classify_byname(const char *name)
 		return &tki_file;
 	}
 
-	if (!illegal_packagename(name, NULL) && !strchr(name, '_'))
+	if (!pkg_name_is_illegal(name, NULL) && !strchr(name, '_'))
 		return &tki_explicit;
 
 invalid:
 	return &tki_unknown;
 }
 
-/* Calling sequence is:
+/*
+ * Calling sequence is:
  *  trig_activate_start(triggername)
  *  dtki->activate_awaiter(awaiting_package) } zero or more times
  *  dtki->activate_awaiter(0)                }  in any order
@@ -293,7 +300,7 @@ trig_activate_start(const char *name)
 	dtki->activate_start();
 }
 
-/*---------- unknown trigger kinds ----------*/
+/*---------- Unknown trigger kinds. ----------*/
 
 static void
 trk_unknown_activate_start(void)
@@ -325,7 +332,7 @@ static const struct trigkindinfo tki_unknown = {
 	.interest_change = trk_unknown_interest_change,
 };
 
-/*---------- explicit triggers ----------*/
+/*---------- Explicit triggers. ----------*/
 
 static FILE *trk_explicit_f;
 static struct varbuf trk_explicit_fn;
@@ -345,10 +352,10 @@ trk_explicit_start(const char *trig)
 {
 	trk_explicit_activate_done();
 
-	varbufreset(&trk_explicit_fn);
-	varbufaddstr(&trk_explicit_fn, triggersdir);
-	varbufaddstr(&trk_explicit_fn, trig);
-	varbufaddc(&trk_explicit_fn, 0);
+	varbuf_reset(&trk_explicit_fn);
+	varbuf_add_str(&trk_explicit_fn, triggersdir);
+	varbuf_add_str(&trk_explicit_fn, trig);
+	varbuf_end_str(&trk_explicit_fn);
 
 	trk_explicit_f = fopen(trk_explicit_fn.buf, "r");
 	if (!trk_explicit_f) {
@@ -386,11 +393,12 @@ trk_explicit_activate_awaiter(struct pkginfo *aw)
 		        trk_explicit_fn.buf);
 
 	while (trk_explicit_fgets(buf, sizeof(buf)) >= 0) {
-		if ((emsg = illegal_packagename(buf, NULL)))
+		emsg = pkg_name_is_illegal(buf, NULL);
+		if (emsg)
 			ohshit(_("trigger interest file `%.250s' syntax error; "
 			         "illegal package name `%.250s': %.250s"),
 			       trk_explicit_fn.buf, buf, emsg);
-		pend = findpackage(buf);
+		pend = pkg_db_find(buf);
 		trig_record_activation(pend, aw, trk_explicit_trig);
 	}
 }
@@ -403,9 +411,8 @@ trk_explicit_interest_change(const char *trig,  struct pkginfo *pkg, int signum)
 	FILE *nf;
 
 	trk_explicit_start(trig);
-	varbufreset(&newfn);
-	varbufprintf(&newfn, "%s/%s.new", triggersdir, trig);
-	varbufaddc(&newfn, 0);
+	varbuf_reset(&newfn);
+	varbuf_printf(&newfn, "%s/%s.new", triggersdir, trig);
 
 	nf = fopen(newfn.buf, "w");
 	if (!nf)
@@ -449,7 +456,7 @@ static const struct trigkindinfo tki_explicit = {
 	.interest_change = trk_explicit_interest_change,
 };
 
-/*---------- file triggers ----------*/
+/*---------- File triggers. ----------*/
 
 static struct {
 	struct trigfileint *head, *tail;
@@ -457,14 +464,15 @@ static struct {
 
 /*
  * Values:
- *  -1: not read,
- *   0: not edited
- *   1: edited
+ *  -1: Not read.
+ *   0: Not edited.
+ *   1: Edited
  */
 static int filetriggers_edited = -1;
 
-/* Called by various people with signum -1 and +1 to mean remove and add
- * and also by trig_file_interests_ensure with signum +2 meaning add
+/*
+ * Called by various people with signum -1 and +1 to mean remove and add
+ * and also by trig_file_interests_ensure() with signum +2 meaning add
  * but die if already present.
  */
 static void
@@ -485,7 +493,7 @@ trk_file_interest_change(const char *trig, struct pkginfo *pkg, int signum)
 		if (tfi->pkg == pkg)
 			goto found;
 
-	/* not found */
+	/* Not found. */
 	if (signum < 0)
 		return;
 
@@ -505,7 +513,7 @@ found:
 	if (signum > 0)
 		return;
 
-	/* remove it: */
+	/* Remove it: */
 	*search = tfi->samefile_next;
 	LIST_UNLINK_PART(filetriggers, tfi, inoverall.);
 edited:
@@ -580,11 +588,13 @@ trig_file_interests_ensure(void)
 			ohshit(_("syntax error in file triggers file `%.250s'"),
 			       triggersfilefile);
 		*space++ = '\0';
-		if ((emsg = illegal_packagename(space, NULL)))
+
+		emsg = pkg_name_is_illegal(space, NULL);
+		if (emsg)
 			ohshit(_("file triggers record mentions illegal "
 			         "package name `%.250s' (for interest in file "
 				 "`%.250s'): %.250s"), space, linebuf, emsg);
-		pkg = findpackage(space);
+		pkg = pkg_db_find(space);
 		trk_file_interest_change(linebuf, pkg, +2);
 	}
 	pop_cleanup(ehflag_normaltidy);
@@ -639,7 +649,7 @@ static const struct trigkindinfo tki_file = {
 	.interest_change = trk_file_interest_change,
 };
 
-/*---------- trigger control info file ----------*/
+/*---------- Trigger control info file. ----------*/
 
 static void
 trig_cicb_interest_change(const char *trig, struct pkginfo *pkg, int signum)
@@ -678,7 +688,7 @@ parse_ci_call(const char *file, const char *cmd, trig_parse_cicb *cb,
 {
 	const char *emsg;
 
-	emsg = illegal_triggername(trig);
+	emsg = trig_name_is_illegal(trig);
 	if (emsg)
 		ohshit(_("triggers ci file `%.250s' contains illegal trigger "
 		         "syntax in trigger name `%.250s': %.250s"),
@@ -698,8 +708,7 @@ trig_parse_ci(const char *file, trig_parse_cicb *interest,
 	f = fopen(file, "r");
 	if (!f) {
 		if (errno == ENOENT)
-			/* No file is just like an empty one. */
-			return;
+			return; /* No file is just like an empty one. */
 		ohshite(_("unable to open triggers ci file `%.250s'"), file);
 	}
 	push_cleanup(cu_closefile, ~0, NULL, 0, 1, f);
@@ -728,10 +737,10 @@ trig_parse_ci(const char *file, trig_parse_cicb *interest,
 			       cmd);
 		}
 	}
-	pop_cleanup(ehflag_normaltidy); /* fclose */
+	pop_cleanup(ehflag_normaltidy); /* fclose() */
 }
 
-/*---------- Unincorp file incorporation ----------*/
+/*---------- Unincorp file incorporation. ----------*/
 
 static void
 tdm_incorp_trig_begin(const char *trig)
@@ -742,7 +751,7 @@ tdm_incorp_trig_begin(const char *trig)
 static void
 tdm_incorp_package(const char *awname)
 {
-	struct pkginfo *aw = strcmp(awname, "-") ? findpackage(awname) : NULL;
+	struct pkginfo *aw = strcmp(awname, "-") ? pkg_db_find(awname) : NULL;
 
 	dtki->activate_awaiter(aw);
 }
@@ -760,13 +769,13 @@ static const struct trigdefmeths tdm_incorp = {
 };
 
 void
-trig_incorporate(enum modstatdb_rw cstatus, const char *admindir)
+trig_incorporate(enum modstatdb_rw cstatus)
 {
-	int ur;
+	enum trigdef_update_status ur;
 	enum trigdef_updateflags tduf;
 
 	free(triggersdir);
-	triggersdir = trig_get_triggersdir(admindir);
+	triggersdir = dpkg_db_get_path(TRIGGERSDIR);
 
 	free(triggersfilefile);
 	triggersfilefile = trig_get_filename(triggersdir, "File");
@@ -784,8 +793,8 @@ trig_incorporate(enum modstatdb_rw cstatus, const char *admindir)
 			tduf |= tduf_writeifenoent;
 	}
 
-	ur = trigdef_update_start(tduf, admindir);
-	if (ur == -1 && cstatus >= msdbrw_write) {
+	ur = trigdef_update_start(tduf);
+	if (ur == tdus_error_no_dir && cstatus >= msdbrw_write) {
 		if (mkdir(triggersdir, 0755)) {
 			if (errno != EEXIST)
 				ohshite(_("unable to create triggers state"
@@ -794,20 +803,20 @@ trig_incorporate(enum modstatdb_rw cstatus, const char *admindir)
 			ohshite(_("unable to set ownership of triggers state"
 			          " directory `%.250s'"), triggersdir);
 		}
-		ur = trigdef_update_start(tduf, admindir);
+		ur = trigdef_update_start(tduf);
 	}
 	switch (ur) {
-	case -2:
+	case tdus_error_empty_deferred:
 		return;
-	case -1:
-	case -3:
+	case tdus_error_no_dir:
+	case tdus_error_no_deferred:
 		if (!trigh.transitional_activate)
 			return;
 	/* Fall through. */
-	case 1:
+	case tdus_no_deferred:
 		trigh.transitional_activate(cstatus);
 		break;
-	case 2:
+	case tdus_ok:
 		/* Read and incorporate triggers. */
 		trigdef_parse();
 		break;
@@ -819,7 +828,7 @@ trig_incorporate(enum modstatdb_rw cstatus, const char *admindir)
 	trigdef_process_done();
 }
 
-/*---------- default hooks ----------*/
+/*---------- Default hooks. ----------*/
 
 struct filenamenode {
 	struct filenamenode *next;
@@ -837,6 +846,7 @@ th_simple_nn_find(const char *name, bool nonew)
 	for (search = trigger_files; search; search = search->next)
 		if (!strcmp(search->name, name))
 			return search;
+
 	/* Not found. */
 	if (nonew)
 		return NULL;
