@@ -39,18 +39,17 @@
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
 #include <dpkg/file.h>
+#include <dpkg/glob.h>
 #include <dpkg/buffer.h>
 #include <dpkg/myopt.h>
 
-#include "glob.h"
 #include "filesdb.h"
 
 
 const char thisname[] = "dpkg-divert";
 const char printforhelp[] = N_("Use --help for help about querying packages.");
 
-const struct cmdinfo *cipaction = NULL;
-const char *admindir = ADMINDIR;
+static const char *admindir;
 
 static bool opt_pkgname_match_any = true;
 static const char *opt_pkgname = NULL;
@@ -158,18 +157,18 @@ file_stat(struct file *f)
 static void
 check_writable_dir(struct file *f)
 {
-	struct varbuf tmpname = VARBUF_INIT;
+	char *tmpname;
 	int tmpfd;
 
-	varbufprintf(&tmpname, "%s%s", f->name, ".dpkg-divert.tmp");
+	m_asprintf(&tmpname, "%s%s", f->name, ".dpkg-divert.tmp");
 
-	tmpfd = creat(tmpname.buf, 0600);
+	tmpfd = creat(tmpname, 0600);
 	if (tmpfd < 0)
 		ohshite(_("error checking '%s'"), f->name);
 	close(tmpfd);
-	unlink(tmpname.buf);
+	unlink(tmpname);
 
-	varbuf_destroy(&tmpname);
+	free(tmpname);
 }
 
 static bool
@@ -207,18 +206,20 @@ check_rename(struct file *src, struct file *dst)
 	return true;
 }
 
-static int
-file_copy(const char *src, const char *dst)
+static void
+file_copy(const char *src, const char *realdst)
 {
+	char *dst;
 	int srcfd, dstfd;
 
 	srcfd = open(src, O_RDONLY);
 	if (srcfd < 0)
-		return -1;
+		ohshite(_("unable to open file '%s'"), src);
 
+	m_asprintf(&dst, "%s%s", realdst, ".dpkg-divert.tmp");
 	dstfd = creat(dst, 0600);
 	if (dstfd < 0)
-		return -1;
+		ohshite(_("unable to create file '%s'"), dst);
 
 	/* FIXME: leaves a dangling destination file on error. */
 
@@ -227,36 +228,16 @@ file_copy(const char *src, const char *dst)
 	close(srcfd);
 
 	if (fsync(dstfd))
-		return -1;
+		ohshite(_("unable to sync file '%s'"), dst);
 	if (close(dstfd))
-		return -1;
+		ohshite(_("unable to close file '%s'"), dst);
 
 	file_copy_perms(src, dst);
 
-	return 0;
-}
+	if (rename(dst, realdst) != 0)
+		ohshite(_("cannot rename '%s' to '%s'"), dst, realdst);
 
-static int
-rename_mv(const char *src, const char *dst)
-{
-	struct varbuf tmpdst = VARBUF_INIT;
-
-	if (rename(src, dst) == 0)
-		return 0;
-
-	varbufprintf(&tmpdst, "%s%s", dst, ".dpkg-divert.tmp");
-
-	/* If a simple rename didn't work try an atomic copy, rename, unlink
-	 * instead. */
-	if (file_copy(src, tmpdst.buf) != 0)
-		return -1;
-
-	if (rename(tmpdst.buf, dst) != 0)
-		return -1;
-
-	varbuf_destroy(&tmpdst);
-
-	return 0;
+	free(dst);
 }
 
 static void
@@ -270,9 +251,15 @@ file_rename(struct file *src, struct file *dst)
 			ohshite(_("rename: remove duplicate old link '%s'"),
 			        src->name);
 	} else {
-		if (rename_mv(src->name, dst->name))
-			ohshite(_("cannot rename '%s' to '%s'"),
-			        src->name, dst->name);
+		if (rename(src->name, dst->name) == 0)
+			return;
+
+		/* If a rename didn't work try moving the file instead. */
+		file_copy(src->name, dst->name);
+
+		if (unlink(src->name))
+			ohshite(_("unable to remove copied source file '%s'"),
+			        src->name);
 	}
 }
 
@@ -289,21 +276,21 @@ static const char *
 varbuf_diversion(struct varbuf *str, const char *pkgname,
                  const char *filename, const char *divertto)
 {
-	varbufreset(str);
+	varbuf_reset(str);
 
 	if (pkgname == NULL) {
 		if (divertto == NULL)
-			varbufprintf(str, _("local diversion of %s"), filename);
+			varbuf_printf(str, _("local diversion of %s"), filename);
 		else
-			varbufprintf(str, _("local diversion of %s to %s"),
-			             filename, divertto);
+			varbuf_printf(str, _("local diversion of %s to %s"),
+			              filename, divertto);
 	} else {
 		if (divertto == NULL)
-			varbufprintf(str, _("diversion of %s by %s"),
-			             filename, pkgname);
+			varbuf_printf(str, _("diversion of %s by %s"),
+			              filename, pkgname);
 		else
-			varbufprintf(str, _("diversion of %s to %s by %s"),
-			             filename, divertto, pkgname);
+			varbuf_printf(str, _("diversion of %s to %s by %s"),
+			              filename, divertto, pkgname);
 	}
 
 	return str->buf;
@@ -315,13 +302,13 @@ diversion_current(const char *filename)
 	static struct varbuf str = VARBUF_INIT;
 
 	if (opt_pkgname_match_any) {
-		varbufreset(&str);
+		varbuf_reset(&str);
 
 		if (opt_divertto == NULL)
-			varbufprintf(&str, _("any diversion of %s"), filename);
+			varbuf_printf(&str, _("any diversion of %s"), filename);
 		else
-			varbufprintf(&str, _("any diversion of %s to %s"),
-			             filename, opt_divertto);
+			varbuf_printf(&str, _("any diversion of %s to %s"),
+			              filename, opt_divertto);
 	} else {
 		return varbuf_diversion(&str, opt_pkgname, filename, opt_divertto);
 	}
@@ -355,21 +342,19 @@ diversion_describe(struct diversion *d)
 static void
 divertdb_write(void)
 {
+	char *dbname, *dbname_new, *dbname_old;
 	FILE *dbfile;
 	struct fileiterator *iter;
 	struct filenamenode *namenode;
-	struct varbuf dbname = VARBUF_INIT;
-	struct varbuf dbname_new = VARBUF_INIT;
-	struct varbuf dbname_old = VARBUF_INIT;
 
-	varbufprintf(&dbname, "%s/%s", admindir, DIVERSIONSFILE);
-	varbufprintf(&dbname_new, "%s%s", dbname.buf, NEWDBEXT);
-	varbufprintf(&dbname_old, "%s%s", dbname.buf, OLDDBEXT);
+	dbname = dpkg_db_get_path(DIVERSIONSFILE);
+	m_asprintf(&dbname_new, "%s%s", dbname, NEWDBEXT);
+	m_asprintf(&dbname_old, "%s%s", dbname, OLDDBEXT);
 
-	dbfile = fopen(dbname_new.buf, "w");
+	dbfile = fopen(dbname_new, "w");
 	if (!dbfile)
 		ohshite(_("cannot create new %s file"), DIVERSIONSFILE);
-	chmod(dbname_new.buf, 0644);
+	chmod(dbname_new, 0644);
 
 	iter = iterfilestart();
 	while ((namenode = iterfilenext(iter))) {
@@ -386,22 +371,22 @@ divertdb_write(void)
 	iterfileend(iter);
 
 	if (fflush(dbfile))
-		ohshite(_("unable to flush file '%s'"), dbname_new.buf);
+		ohshite(_("unable to flush file '%s'"), dbname_new);
 	if (fsync(fileno(dbfile)))
-		ohshite(_("unable to sync file '%s'"), dbname_new.buf);
+		ohshite(_("unable to sync file '%s'"), dbname_new);
 	if (fclose(dbfile))
-		ohshite(_("unable to close file '%s'"), dbname_new.buf);
+		ohshite(_("unable to close file '%s'"), dbname_new);
 
-	if (unlink(dbname_old.buf) && errno != ENOENT)
+	if (unlink(dbname_old) && errno != ENOENT)
 		ohshite(_("error removing old diversions-old"));
-	if (link(dbname.buf, dbname_old.buf) && errno != ENOENT)
+	if (link(dbname, dbname_old) && errno != ENOENT)
 		ohshite(_("error creating new diversions-old"));
-	if (rename(dbname_new.buf, dbname.buf))
+	if (rename(dbname_new, dbname))
 		ohshite(_("error installing new diversions"));
 
-	varbuf_destroy(&dbname);
-	varbuf_destroy(&dbname_new);
-	varbuf_destroy(&dbname_old);
+	free(dbname);
+	free(dbname_new);
+	free(dbname_old);
 }
 
 static int
@@ -435,10 +420,10 @@ diversion_add(const char *const *argv)
 
 	/* Handle divertto. */
 	if (opt_divertto == NULL) {
-		struct varbuf str = VARBUF_INIT;
+		char *str;
 
-		varbufprintf(&str, "%s.distrib", filename);
-		opt_divertto = varbuf_detach(&str);
+		m_asprintf(&str, "%s.distrib", filename);
+		opt_divertto = str;
 	}
 	if (opt_divertto[0] != '/')
 		badusage(_("filename \"%s\" is not absolute"), opt_divertto);
@@ -454,7 +439,7 @@ diversion_add(const char *const *argv)
 	if (opt_pkgname == NULL)
 		pkg = NULL;
 	else
-		pkg = findpackage(opt_pkgname);
+		pkg = pkg_db_find(opt_pkgname);
 
 	/* Check we are not stomping over an existing diversion. */
 	if (fnn_from->divert || fnn_to->divert) {
@@ -529,7 +514,7 @@ diversion_remove(const char *const *argv)
 	if (opt_pkgname == NULL)
 		pkg = NULL;
 	else
-		pkg = findpackage(opt_pkgname);
+		pkg = pkg_db_find(opt_pkgname);
 
 	contest = namenode->divert;
 	altname = contest->useinstead->divert;
@@ -659,22 +644,6 @@ diversion_listpackage(const char *const *argv)
 	return 0;
 }
 
-static inline int
-option_short(int c)
-{
-	return c ? c : '\b';
-}
-
-static void
-setaction(const struct cmdinfo *cip, const char *value)
-{
-	if (cipaction)
-		badusage(_("conflicting actions -%c (--%s) and -%c (--%s)"),
-		         option_short(cip->oshort), cip->olong,
-		         option_short(cipaction->oshort), cipaction->olong);
-	cipaction = cip;
-}
-
 static void
 setpackage(const struct cmdinfo *cip, const char *value)
 {
@@ -696,9 +665,6 @@ setdivertto(const struct cmdinfo *cip, const char *value)
 		badusage(_("divert-to may not contain newlines"));
 }
 
-#define ACTION(longopt, shortopt, code, function) \
- { longopt, shortopt, 0, 0, 0, setaction, code, 0, (voidfnp)function }
-
 static const struct cmdinfo cmdinfo_add =
 	ACTION("add",         0, 0, diversion_add);
 
@@ -724,28 +690,31 @@ static const struct cmdinfo cmdinfos[] = {
 int
 main(int argc, const char * const *argv)
 {
-	jmp_buf ejbuf;
-	int (*actionfunction)(const char *const *argv);
+	const char *env_pkgname;
 	int ret;
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 
-	standard_startup(&ejbuf);
+	standard_startup();
 	myopt(&argv, cmdinfos);
 
-	if (!cipaction)
-		cipaction = &cmdinfo_add;
+	admindir = dpkg_db_set_dir(admindir);
 
-	actionfunction = (int (*)(const char *const *))cipaction->farg;
+	env_pkgname = getenv("DPKG_MAINTSCRIPT_PACKAGE");
+	if (opt_pkgname_match_any && env_pkgname)
+		setpackage(NULL, env_pkgname);
+
+	if (!cipaction)
+		setaction(&cmdinfo_add, NULL);
 
 	setvbuf(stdout, NULL, _IONBF, 0);
 
 	filesdbinit();
 	ensure_diversions();
 
-	ret = actionfunction(argv);
+	ret = cipaction->action(argv);
 
 	standard_shutdown();
 
