@@ -46,6 +46,7 @@
 #include <dpkg/i18n.h>
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
+#include <dpkg/subproc.h>
 #include <dpkg/command.h>
 #include <dpkg/myopt.h>
 
@@ -66,10 +67,13 @@ printversion(const struct cmdinfo *ci, const char *value)
 
   exit(0);
 }
+
 /*
-   options that need fixing:
-  dpkg --yet-to-unpack                 \n\
-  */
+ * FIXME: Options that need fixing:
+ * dpkg --yet-to-unpack
+ * dpkg --command-fd
+ */
+
 static void DPKG_ATTR_NORET
 usage(const struct cmdinfo *ci, const char *value)
 {
@@ -162,7 +166,7 @@ usage(const struct cmdinfo *ci, const char *value)
 }
 
 const char thisname[]= "dpkg";
-const char architecture[]= ARCHITECTURE;
+const char native_arch[] = ARCHITECTURE;
 const char printforhelp[]= N_(
 "Type dpkg --help for help about installing and deinstalling packages [*];\n"
 "Use `dselect' or `aptitude' for user-friendly package management;\n"
@@ -172,12 +176,9 @@ const char printforhelp[]= N_(
 "\n"
 "Options marked [*] produce a lot of output - pipe it through `less' or `more' !");
 
-const struct cmdinfo *cipaction = NULL;
 int f_pending=0, f_recursive=0, f_alsoselect=1, f_skipsame=0, f_noact=0;
 int f_autodeconf=0, f_nodebsig=0;
 int f_triggers = 0;
-unsigned long f_debug=0;
-/* Change fc_overwrite to 1 to enable force-overwrite by default */
 int fc_downgrade=1, fc_configureany=0, fc_hold=0, fc_removereinstreq=0, fc_overwrite=0;
 int fc_removeessential=0, fc_conflicts=0, fc_depends=0, fc_dependsversion=0;
 int fc_breaks=0, fc_badpath=0, fc_overwritediverted=0, fc_architecture=0;
@@ -188,94 +189,146 @@ int fc_unsafe_io = 0;
 int fc_badverify = 0;
 
 int errabort = 50;
-const char *admindir= ADMINDIR;
+static const char *admindir = ADMINDIR;
 const char *instdir= "";
 struct pkg_list *ignoredependss = NULL;
 
+static const char *
+forcetype_str(char type)
+{
+  switch (type) {
+  case '\0':
+  case ' ':
+    return "   ";
+  case '*':
+    return "[*]";
+  case '!':
+    return "[!]";
+  default:
+    internerr("unknown force type %c", type);
+  }
+}
+
 static const struct forceinfo {
   const char *name;
   int *opt;
+  char type;
+  const char *desc;
 } forceinfos[]= {
-  { "downgrade",           &fc_downgrade                },
-  { "configure-any",       &fc_configureany             },
-  { "hold",                &fc_hold                     },
-  { "remove-reinstreq",    &fc_removereinstreq          },
-  { "remove-essential",    &fc_removeessential          },
-  { "conflicts",           &fc_conflicts                },
-  { "confnew",             &fc_conff_new                },
-  { "confold",             &fc_conff_old                },
-  { "confdef",             &fc_conff_def                },
-  { "confmiss",            &fc_conff_miss               },
-  { "confask",             &fc_conff_ask                },
-  { "depends",             &fc_depends                  },
-  { "depends-version",     &fc_dependsversion           },
-  { "breaks",              &fc_breaks                   },
-  { "bad-path",            &fc_badpath                  },
-  { "not-root",            &fc_nonroot                  },
-  { "overwrite",           &fc_overwrite                },
-  { "overwrite-diverted",  &fc_overwritediverted        },
-  { "overwrite-dir",       &fc_overwritedir             },
-  { "unsafe-io",           &fc_unsafe_io                },
-  { "architecture",        &fc_architecture             },
-  { "bad-verify",          &fc_badverify                },
-  {  NULL                                               }
+  { "all",                 NULL,
+    '!', N_("Set all force options")},
+  { "downgrade",           &fc_downgrade,
+    '*', N_("Replace a package with a lower version") },
+  { "configure-any",       &fc_configureany,
+    ' ', N_("Configure any package which may help this one") },
+  { "hold",                &fc_hold,
+    ' ', N_("Process incidental packages even when on hold") },
+  { "not-root",            &fc_nonroot,
+    ' ', N_("Try to (de)install things even when not root") },
+  { "bad-path",            &fc_badpath,
+    ' ', N_("PATH is missing important programs, problems likely") },
+  { "bad-verify",          &fc_badverify,
+    ' ', N_("Install a package even if it fails authenticity check") },
+  { "overwrite",           &fc_overwrite,
+    ' ', N_("Overwrite a file from one package with another") },
+  { "overwrite-diverted",  &fc_overwritediverted,
+    ' ', N_("Overwrite a diverted file with an undiverted version") },
+  { "overwrite-dir",       &fc_overwritedir,
+    '!', N_("Overwrite one package's directory with another's file") },
+  { "unsafe-io",           &fc_unsafe_io,
+    '!', N_("Do not perform safe I/O operations when unpacking") },
+  { "confnew",             &fc_conff_new,
+    '!', N_("Always use the new config files, don't prompt") },
+  { "confold",             &fc_conff_old,
+    '!', N_("Always use the old config files, don't prompt") },
+  /* XXX: Handle automatically line wrappings. */
+  { "confdef",             &fc_conff_def,
+    '!', N_("Use the default option for new config files if one\n"
+"                         is available, don't prompt. If no default can be found,\n"
+"                         you will be prompted unless one of the confold or\n"
+"                         confnew options is also given") },
+  { "confmiss",            &fc_conff_miss,
+    '!', N_("Always install missing config files") },
+  { "confask",             &fc_conff_ask,
+    '!', N_("Offer to replace config files with no new versions") },
+  { "architecture",        &fc_architecture,
+    '!', N_("Process even packages with wrong or no architecture") },
+  { "breaks",              &fc_breaks,
+    '!', N_("Install even if it would break another package") },
+  { "conflicts",           &fc_conflicts,
+    '!', N_("Allow installation of conflicting packages") },
+  { "depends",             &fc_depends,
+    '!', N_("Turn all dependency problems into warnings") },
+  { "depends-version",     &fc_dependsversion,
+    '!', N_("Turn dependency version problems into warnings") },
+  { "remove-reinstreq",    &fc_removereinstreq,
+    '!', N_("Remove packages which require installation") },
+  { "remove-essential",    &fc_removeessential,
+    '!', N_("Remove an essential package") },
+  { NULL }
 };
 
-static void setaction(const struct cmdinfo *cip, const char *value) {
-  if (cipaction)
-    badusage(_("conflicting actions -%c (--%s) and -%c (--%s)"),
-             cip->oshort, cip->olong, cipaction->oshort, cipaction->olong);
-  cipaction= cip;
-}
+#define DBG_DEF(n, d) \
+  { .flag = dbg_##n, .name = #n, .desc = d }
 
-static void setobsolete(const struct cmdinfo *cip, const char *value) {
-  warning(_("obsolete option '--%s'\n"), cip->olong);
-}
+static const struct debuginfo {
+  int flag;
+  const char *name;
+  const char *desc;
+} debuginfos[] = {
+  DBG_DEF(general,         N_("Generally helpful progress information")),
+  DBG_DEF(scripts,         N_("Invocation and status of maintainer scripts")),
+  DBG_DEF(eachfile,        N_("Output for each file processed")),
+  DBG_DEF(eachfiledetail,  N_("Lots of output for each file processed")),
+  DBG_DEF(conff,           N_("Output for each configuration file")),
+  DBG_DEF(conffdetail,     N_("Lots of output for each configuration file")),
+  DBG_DEF(depcon,          N_("Dependencies and conflicts")),
+  DBG_DEF(depcondetail,    N_("Lots of dependencies/conflicts output")),
+  DBG_DEF(triggers,        N_("Trigger activation and processing")),
+  DBG_DEF(triggersdetail,  N_("Lots of output regarding triggers")),
+  DBG_DEF(triggersstupid,  N_("Silly amounts of output regarding triggers")),
+  DBG_DEF(veryverbose,     N_("Lots of drivel about eg the dpkg/info directory")),
+  DBG_DEF(stupidlyverbose, N_("Insane amounts of drivel")),
+  { 0, NULL, NULL }
+};
 
 static void setdebug(const struct cmdinfo *cpi, const char *value) {
   char *endp;
+  unsigned long mask;
+  const struct debuginfo *dip;
 
   if (*value == 'h') {
     printf(_(
 "%s debugging option, --debug=<octal> or -D<octal>:\n"
 "\n"
-" number  ref. in source   description\n"
-"      1   general           Generally helpful progress information\n"
-"      2   scripts           Invocation and status of maintainer scripts\n"
-"     10   eachfile          Output for each file processed\n"
-"    100   eachfiledetail    Lots of output for each file processed\n"
-"     20   conff             Output for each configuration file\n"
-"    200   conffdetail       Lots of output for each configuration file\n"
-"     40   depcon            Dependencies and conflicts\n"
-"    400   depcondetail      Lots of dependencies/conflicts output\n"
-"  10000   triggers          Trigger activation and processing\n"
-"  20000   triggersdetail    Lots of output regarding triggers\n"
-"  40000   triggersstupid    Silly amounts of output regarding triggers\n"
-"   1000   veryverbose       Lots of drivel about eg the dpkg/info directory\n"
-"   2000   stupidlyverbose   Insane amounts of drivel\n"
-"\n"
-"Debugging options are be mixed using bitwise-or.\n"
-"Note that the meanings and values are subject to change.\n"), DPKG);
+" Number  Ref. in source   Description\n"), DPKG);
+
+    for (dip = debuginfos; dip->name; dip++)
+      printf(" %6o  %-16s %s\n", dip->flag, dip->name, gettext(dip->desc));
+
+    printf(_("\n"
+"Debugging options can be mixed using bitwise-or.\n"
+"Note that the meanings and values are subject to change.\n"));
     m_output(stdout, _("<standard output>"));
     exit(0);
   }
-  
-  f_debug= strtoul(value,&endp,8);
+
+  mask = strtoul(value, &endp, 8);
   if (value == endp || *endp) badusage(_("--debug requires an octal argument"));
+
+  debug_set_mask(mask);
 }
 
 static void
 setfilter(const struct cmdinfo *cip, const char *value)
 {
-  filter_add(value, cip->arg);
+  filter_add(value, cip->arg_int);
 }
 
 static void setroot(const struct cmdinfo *cip, const char *value) {
   char *p;
   instdir= value;
-  p= m_malloc(strlen(value) + sizeof(ADMINDIR));
-  strcpy(p,value);
-  strcat(p,ADMINDIR);
+  m_asprintf(&p, "%s%s", value, ADMINDIR);
   admindir= p;
 }
 
@@ -295,11 +348,11 @@ static void ignoredepends(const struct cmdinfo *cip, const char *value) {
   }
   p= copy;
   while (*p) {
-    pnerr = illegal_packagename(p, NULL);
+    pnerr = pkg_name_is_illegal(p, NULL);
     if (pnerr) ohshite(_("--ignore-depends requires a legal package name. "
                        "`%.250s' is not; %s"), p, pnerr);
 
-    pkg_list_prepend(&ignoredependss, findpackage(p));
+    pkg_list_prepend(&ignoredependss, pkg_db_find(p));
 
     p+= strlen(p)+1;
   }
@@ -318,8 +371,6 @@ static void setinteger(const struct cmdinfo *cip, const char *value) {
 }
 
 static void setpipe(const struct cmdinfo *cip, const char *value) {
-  struct pipef **pipe_head = cip->parg;
-  struct pipef *pipe_new;
   unsigned long v;
   char *ep;
 
@@ -327,12 +378,7 @@ static void setpipe(const struct cmdinfo *cip, const char *value) {
   if (value == ep || *ep || v > INT_MAX)
     badusage(_("invalid integer for --%s: `%.250s'"),cip->olong,value);
 
-  setcloexec(v, _("<package status and progress file descriptor>"));
-
-  pipe_new = nfmalloc(sizeof(struct pipef));
-  pipe_new->fd = v;
-  pipe_new->next = *pipe_head;
-  *pipe_head = pipe_new;
+  statusfd_add(v);
 }
 
 static bool
@@ -355,11 +401,13 @@ struct invoke_hook *pre_invoke_hooks = NULL;
 struct invoke_hook **pre_invoke_hooks_tail = &pre_invoke_hooks;
 struct invoke_hook *post_invoke_hooks = NULL;
 struct invoke_hook **post_invoke_hooks_tail = &post_invoke_hooks;
+struct invoke_hook *status_loggers = NULL;
+struct invoke_hook **status_loggers_tail = &status_loggers;
 
 static void
 set_invoke_hook(const struct cmdinfo *cip, const char *value)
 {
-  struct invoke_hook ***hook_tail = cip->parg;
+  struct invoke_hook ***hook_tail = cip->arg_ptr;
   struct invoke_hook *hook_new;
 
   hook_new = nfmalloc(sizeof(struct invoke_hook));
@@ -392,6 +440,34 @@ run_invoke_hooks(const char *action, struct invoke_hook *hook_head)
   unsetenv("DPKG_HOOK_ACTION");
 }
 
+static void
+run_status_loggers(struct invoke_hook *hook_head)
+{
+  struct invoke_hook *hook;
+
+  for (hook = hook_head; hook; hook = hook->next) {
+    pid_t pid;
+    int p[2];
+
+    m_pipe(p);
+
+    pid = subproc_fork();
+    if (pid == 0) {
+      /* Setup stdin and stdout. */
+      m_dup2(p[0], 0);
+      close(1);
+
+      close(p[0]);
+      close(p[1]);
+
+      command_shell(hook->command, _("status logger"));
+    }
+    close(p[0]);
+
+    statusfd_add(p[1]);
+  }
+}
+
 static void setforce(const struct cmdinfo *cip, const char *value) {
   const char *comma;
   size_t l;
@@ -402,36 +478,16 @@ static void setforce(const struct cmdinfo *cip, const char *value) {
 "%s forcing options - control behaviour when problems found:\n"
 "  warn but continue:  --force-<thing>,<thing>,...\n"
 "  stop with error:    --refuse-<thing>,<thing>,... | --no-force-<thing>,...\n"
-" Forcing things:\n"
-"  all [!]                Set all force options\n"
-"  downgrade [*]          Replace a package with a lower version\n"
-"  configure-any          Configure any package which may help this one\n"
-"  hold                   Process incidental packages even when on hold\n"
-"  bad-path               PATH is missing important programs, problems likely\n"
-"  not-root               Try to (de)install things even when not root\n"
-"  overwrite              Overwrite a file from one package with another\n"
-"  overwrite-diverted     Overwrite a diverted file with an undiverted version\n"
-"  bad-verify             Install a package even if it fails authenticity check\n"
-"  depends-version [!]    Turn dependency version problems into warnings\n"
-"  depends [!]            Turn all dependency problems into warnings\n"
-"  confnew [!]            Always use the new config files, don't prompt\n"
-"  confold [!]            Always use the old config files, don't prompt\n"
-"  confdef [!]            Use the default option for new config files if one\n"
-"                         is available, don't prompt. If no default can be found,\n"
-"                         you will be prompted unless one of the confold or\n"
-"                         confnew options is also given\n"
-"  confmiss [!]           Always install missing config files\n"
-"  confask [!]            Offer to replace config files with no new versions\n"
-"  breaks [!]             Install even if it would break another package\n"
-"  conflicts [!]          Allow installation of conflicting packages\n"
-"  architecture [!]       Process even packages with wrong architecture\n"
-"  overwrite-dir [!]      Overwrite one package's directory with another's file\n"
-"  unsafe-io [!]          Do not perform safe I/O operations when unpacking\n"
-"  remove-reinstreq [!]   Remove packages which require installation\n"
-"  remove-essential [!]   Remove an essential package\n"
+" Forcing things:\n"), DPKG);
+
+    for (fip = forceinfos; fip->name; fip++)
+      printf("  %s %-18s %s\n", forcetype_str(fip->type), fip->name,
+             gettext(fip->desc));
+
+    printf(_(
 "\n"
 "WARNING - use of options marked [!] can seriously damage your installation.\n"
-"Forcing options marked [*] are enabled by default.\n"), DPKG);
+"Forcing options marked [*] are enabled by default.\n"));
     m_output(stdout, _("<standard output>"));
     exit(0);
   }
@@ -441,38 +497,34 @@ static void setforce(const struct cmdinfo *cip, const char *value) {
     l = comma ? (size_t)(comma - value) : strlen(value);
     for (fip=forceinfos; fip->name; fip++)
       if (!strncmp(fip->name,value,l) && strlen(fip->name)==l) break;
+
     if (!fip->name) {
-      if (strncmp("all", value, l) == 0) {
-	for (fip=forceinfos; fip->name; fip++)
-	  if (fip->opt)
-	    *fip->opt= cip->arg;
-      } else
-	badusage(_("unknown force/refuse option `%.*s'"),
-	         (int)min(l, 250), value);
+      badusage(_("unknown force/refuse option `%.*s'"),
+               (int)min(l, 250), value);
+    } else if (strcmp(fip->name, "all") == 0) {
+      for (fip = forceinfos; fip->name; fip++)
+        if (fip->opt)
+          *fip->opt = cip->arg_int;
+    } else if (fip->opt) {
+      *fip->opt = cip->arg_int;
     } else {
-      if (fip->opt)
-	*fip->opt= cip->arg;
-      else
-	warning(_("obsolete force/refuse option '%s'\n"), fip->name);
+      warning(_("obsolete force/refuse option '%s'\n"), fip->name);
     }
+
     if (!comma) break;
     value= ++comma;
   }
 }
 
-void execbackend(const char *const *argv) DPKG_ATTR_NORET;
-void commandfd(const char *const *argv);
+int execbackend(const char *const *argv) DPKG_ATTR_NORET;
+int commandfd(const char *const *argv);
+
+/* This table has both the action entries in it and the normal options.
+ * The action entries are made with the ACTION macro, as they all
+ * have a very similar structure. */
 static const struct cmdinfo cmdinfos[]= {
-  /* This table has both the action entries in it and the normal options.
-   * The action entries are made with the ACTION macro, as they all
-   * have a very similar structure.
-   */
-#define ACTION(longopt,shortopt,code,function) \
- { longopt, shortopt, 0, NULL, NULL, setaction, code, NULL, (voidfnp)function }
-#define OBSOLETE(longopt,shortopt) \
- { longopt, shortopt, 0, NULL, NULL, setobsolete, 0, NULL, NULL }
 #define ACTIONBACKEND(longopt, shortopt, backend) \
- { longopt, shortopt, 0, NULL, NULL, setaction, 0, (void *)backend, (voidfnp)execbackend }
+ { longopt, shortopt, 0, NULL, NULL, setaction, 0, (void *)backend, execbackend }
 
   ACTION( "install",                        'i', act_install,              archivefiles    ),
   ACTION( "unpack",                          0,  act_unpack,               archivefiles    ),
@@ -506,12 +558,13 @@ static const struct cmdinfo cmdinfos[]= {
 /*
   ACTION( "command-fd",                   'c', act_commandfd,   commandfd     ),
 */
-  
+
   { "pre-invoke",        0,   1, NULL,          NULL,      set_invoke_hook, 0, &pre_invoke_hooks_tail },
   { "post-invoke",       0,   1, NULL,          NULL,      set_invoke_hook, 0, &post_invoke_hooks_tail },
   { "path-exclude",      0,   1, NULL,          NULL,      setfilter,     0 },
   { "path-include",      0,   1, NULL,          NULL,      setfilter,     1 },
-  { "status-fd",         0,   1, NULL,          NULL,      setpipe, 0, &status_pipes },
+  { "status-logger",     0,   1, NULL,          NULL,      set_invoke_hook, 0, &status_loggers_tail },
+  { "status-fd",         0,   1, NULL,          NULL,      setpipe, 0 },
   { "log",               0,   1, NULL,          &log_file, NULL,    0 },
   { "pending",           'a', 0, &f_pending,    NULL,      NULL,    1 },
   { "recursive",         'R', 0, &f_recursive,  NULL,      NULL,    1 },
@@ -552,26 +605,16 @@ static const struct cmdinfo cmdinfos[]= {
   { NULL,                0,   0, NULL,          NULL,      NULL,          0 }
 };
 
-void execbackend(const char *const *argv) {
+int
+execbackend(const char *const *argv)
+{
   struct command cmd;
   char *arg;
 
-  command_init(&cmd, cipaction->parg, NULL);
-  command_add_arg(&cmd, cipaction->parg);
+  command_init(&cmd, cipaction->arg_ptr, NULL);
+  command_add_arg(&cmd, cipaction->arg_ptr);
 
-  /*
-   * Special case: dpkg-query takes the --admindir option, and if dpkg itself
-   * was given a different admin directory, we need to pass it along to it.
-   */
-  if (strcmp(cipaction->parg, DPKGQUERY) == 0 &&
-      strcmp(admindir, ADMINDIR) != 0) {
-    arg = m_malloc((strlen("--admindir=") + strlen(admindir) + 1));
-    sprintf(arg, "--admindir=%s", admindir);
-    command_add_arg(&cmd, arg);
-  }
-
-  arg = m_malloc(2 + strlen(cipaction->olong) + 1);
-  sprintf(arg, "--%s", cipaction->olong);
+  m_asprintf(&arg, "--%s", cipaction->olong);
   command_add_arg(&cmd, arg);
 
   /* Exlicitely separate arguments from options as any user-supplied
@@ -582,17 +625,18 @@ void execbackend(const char *const *argv) {
   command_exec(&cmd);
 }
 
-void commandfd(const char *const *argv) {
-  jmp_buf ejbuf;
+int
+commandfd(const char *const *argv)
+{
   struct varbuf linevb = VARBUF_INIT;
   const char * pipein;
   const char **newargs = NULL;
   char *ptr, *endptr;
   FILE *in;
   unsigned long infd;
+  int ret = 0;
   int c, lno, i;
   bool skipchar;
-  void (*actionfunction)(const char *const *argv);
 
   pipein = *argv++;
   if (pipein == NULL)
@@ -606,15 +650,12 @@ void commandfd(const char *const *argv) {
   if ((in= fdopen(infd, "r")) == NULL)
     ohshite(_("couldn't open `%i' for stream"), (int) infd);
 
-  if (setjmp(ejbuf)) { /* expect warning about possible clobbering of argv */
-    error_unwind(ehflag_bombout); exit(2);
-  }
-
   for (;;) {
     bool mode = false;
     int argc= 1;
     lno= 0;
-    push_error_handler(&ejbuf, print_error_fatal, NULL);
+
+    push_error_context();
 
     do { c= getc(in); if (c == '\n') lno++; } while (c != EOF && isspace(c));
     if (c == EOF) break;
@@ -622,20 +663,23 @@ void commandfd(const char *const *argv) {
       do { c= getc(in); if (c == '\n') lno++; } while (c != EOF && c != '\n');
       continue;
     }
-    varbufreset(&linevb);
+    varbuf_reset(&linevb);
     do {
-      varbufaddc(&linevb,c);
+      varbuf_add_char(&linevb, c);
       c= getc(in);
       if (c == '\n') lno++;
-      if (isspace(c)) argc++;  /* This isn't fully accurate, but overestimating can't hurt. */
+
+      /* This isn't fully accurate, but overestimating can't hurt. */
+      if (isspace(c))
+        argc++;
     } while (c != EOF && c != '\n');
     if (c == EOF) ohshit(_("unexpected eof before end of line %d"),lno);
     if (!argc) continue;
-    varbufaddc(&linevb,0);
+    varbuf_end_str(&linevb);
     newargs = m_realloc(newargs, sizeof(const char *) * (argc + 1));
     argc= 1;
     ptr= linevb.buf;
-    endptr= ptr + linevb.used;
+    endptr = ptr + linevb.used + 1;
     skipchar = false;
     while(ptr < endptr) {
       if (skipchar) {
@@ -662,59 +706,63 @@ void commandfd(const char *const *argv) {
     *ptr = '\0';
     newargs[argc++] = NULL;
 
-/* We strdup each argument, but never free it, because the error messages
- * contain references back to these strings.  Freeing them, and reusing
- * the memory, would make those error messages confusing, to say the
- * least.
- */
+    /* We strdup() each argument, but never free it, because the
+     * error messages contain references back to these strings.
+     * Freeing them, and reusing the memory, would make those
+     * error messages confusing, to say the least. */
     for(i=1;i<argc;i++)
       if (newargs[i])
         newargs[i] = m_strdup(newargs[i]);
 
-    cipaction= NULL;
+    setaction(NULL, NULL);
     myopt((const char *const**)&newargs,cmdinfos);
     if (!cipaction) badusage(_("need an action option"));
 
-    actionfunction= (void (*)(const char* const*))cipaction->farg;
-    actionfunction(newargs);
-    set_error_display(NULL, NULL);
-    error_unwind(ehflag_normaltidy);
+    ret |= cipaction->action(newargs);
+
+    pop_error_context(ehflag_normaltidy);
   }
+
+  return ret;
 }
 
-
 int main(int argc, const char *const *argv) {
-  jmp_buf ejbuf;
-  void (*actionfunction)(const char *const *argv);
+  int ret;
 
   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
 
-  standard_startup(&ejbuf);
+  standard_startup();
   loadcfgfile(DPKG, cmdinfos);
   myopt(&argv, cmdinfos);
 
   if (!cipaction) badusage(_("need an action option"));
 
+  admindir = dpkg_db_set_dir(admindir);
+
+  /* Always set environment, to avoid possible security risks. */
+  if (setenv("DPKG_ADMINDIR", admindir, 1) < 0)
+    ohshite(_("unable to setenv for subprocesses"));
+
   if (!f_triggers)
-    f_triggers = (cipaction->arg == act_triggers && *argv) ? -1 : 1;
+    f_triggers = (cipaction->arg_int == act_triggers && *argv) ? -1 : 1;
 
   setvbuf(stdout, NULL, _IONBF, 0);
 
-  if (is_invoke_action(cipaction->arg))
+  if (is_invoke_action(cipaction->arg_int)) {
     run_invoke_hooks(cipaction->olong, pre_invoke_hooks);
+    run_status_loggers(status_loggers);
+  }
 
   filesdbinit();
 
-  actionfunction= (void (*)(const char* const*))cipaction->farg;
+  ret = cipaction->action(argv);
 
-  actionfunction(argv);
-
-  if (is_invoke_action(cipaction->arg))
+  if (is_invoke_action(cipaction->arg_int))
     run_invoke_hooks(cipaction->olong, post_invoke_hooks);
 
   standard_shutdown();
 
-  return reportbroken_retexitstatus();
+  return reportbroken_retexitstatus(ret);
 }
